//    ___
//   |  _\ A free Lemmings clone
//   |   /_  _ _  ___  _   _  ___ 
//   |  || || \ || _ || |_| ||_ -'
//   |__||_||_\_||_  ||_____||___|
//                _| |
//               |___|
//
//   Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>
//     
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream>
#include <ctime>
#include <cstdio>
#include <unistd.h>
#include <getopt.h>
#include <ClanLib/clanlib.h>

#include "levelmap.hh"
#include "window.hh"
#include "playfield.hh"
#include "button_panel.hh"
#include "pingus.hh"
#include "pingu.hh"
#include "globals.hh"

// Only for temporary use, should be replaced with CL_Mouse
#include "mouse.hh"

char* 
PingusMain::get_title()
{
  static char* title;
  
  if (!title) {
    title = new char[1024];
    strcpy(title, "Pingus ");
    strcat(title, VERSION);
    strcat(title, " - A free Lemmings clone");
  }

  return title;
}

// check_ars() checks the command line for options and set the
// corresponding global variables to the set values.
void 
PingusMain::check_args(int argc, char* argv[])
{
  int c; 
  int option_index = 0;
  int display_help = 0;

  struct option long_options[] =
  {
    {"speed",           required_argument, 0, 's'},
    {"data-path",       required_argument, 0, 'p'},
    {"level",           required_argument, 0, 'l'},
    {"help",            no_argument,       0, 'h'},
    {"version",         no_argument,       0, 'v'},
    {"frame-rate",      required_argument, 0, 'f'},
    {"use-dirty-tiles", no_argument,       0, 'd'},
    {"print-fps",       no_argument,       0, 'b'},
    {0, 0, 0, 0}
  };

  cout << "Checking arguments..." << endl; 

  while(true) {
    c = getopt_long(argc, argv, "ds:l:hvf:p:b", long_options, &option_index);
    
    if (c == -1)
      break;

    switch(c) {
    case 0:
      cout << "You should normaly never reach this line." << endl;
      exit(EXIT_FAILURE);
      break;
    case 'b': // -b, --print-fps
      print_fps = true;
      break;
    case 'l': // -l, --level
      levelfile = strdup(optarg);
      break;
    case 's': // -s, --speed
      game_speed = atoi(optarg);
      break;
    case 'f': // -f
      global_frame_rate = atoi(optarg);
      break;
    case 'd': // -d
      use_dirty_tiles = true;
      break;
    case 'p': // -p
      pingus_data_dir = strdup(optarg);
      break;
    default:
      cout << PACKAGE << " " << VERSION << endl;
      cout << "-=-=-=-=-=-=-=-=-=-=-=-=-=-=-" << endl;
      cout <<
	"-h, --help              Displays this screen\n"
	"-p, --data-path path    Set the path to load the data files to `path'\n"
	"-l, --level level       Load the level from file `level'\n"
	"-l file                 Load a custom level from `file'\n"
	"-f INT                  Sets the frame rate of the game\n"
	"-d, --use-dirty-tiles   Activate the new dirty tile mechanism\n"
	"-s INT                  Set the game speed\n"
	"-p DIR                  Set the data path\n"
	"-b, --print-fps         Prints the fps to stdout\n"
	   << endl;
      exit(EXIT_SUCCESS);
      break;
    }
  }
 cout << "correct" << endl;
}

void
PingusMain::intro(void)
{
  // Show a simple startup intro
}

// Loads a level from file and starts playing i
int
PingusMain::play_level(char* filename)
{
  PLF*         plf          = new PLF(filename);
  Window*      game         = new Window();

  // Create GuiObj for Window* game 
  Playfield*   playfield    = new Playfield(plf);
  ButtonPanel* button_panel = new ButtonPanel();
  LevelMap*    level_map    = new LevelMap(plf);

  // Connect the button_panel with the playfield
  playfield->set_buttons(button_panel);

  // Add all objects to the window
  game->add_obj(playfield);
  game->add_obj(button_panel);
  game->add_obj(level_map);

  // Give main controll to the window
  game->main_loop();

  delete          plf;
  delete    playfield;
  delete         game;
  delete button_panel;
  delete    level_map;

  return 1;
}

void
PingusMain::load_data()
{
  cout << "Loading data... ";
  //  resource = new CL_ResourceManager(pingus_scriptfile, pingus_datafile);
  resource = new CL_ResourceManager(pingus_scriptfile);

  Pingu::load_data(resource);
  PinguAction::load_data(resource);
  World::load_data(resource);
  Playfield::load_data(resource);
  ButtonPanel::load_data(resource);
  LevelMap::load_data(resource);
  Entrance::load_data(resource);
  Trap::load_data(resource);

  cout << "loading Exit" << endl;
  Exit::load_data(resource);
  cout << "loading EXIT fini" << endl;
  cout << "succesfull" << endl;
}
  
void
PingusMain::init(int argc, char* argv[])
{
  // Print a nice ansi logo on startup
#ifdef NICE_STARTUP_LOGO
  cout << "[0;1;30;44m         [40m      [44m       [40m"
"[1;44m        [40m        [44m      [2C[47m  [5C      [1C  [3C  [1C  [2C  [1C  [2C  [40m"
"[1;30;44m        [40m [47m  [40m [47m  [40m  [44m      [2C[47m  [7C  [3C   [2C  [1C  [2C  [2C    [40m"
"[1;30;44m        [40m [47m [40m   [47m [40m   [44m     [2C[47m  [7C  [3C    [1C  [1C  [2C  [3C  [40m"
"[1;30;44m        [40m  [43m    [40m   [44m     [2C[47m  [7C  [3C  [1C    [1C  [2C  [2C    [40m"
"[1;30;44m        [40m [0;30;43m`----'[40m  [1;44m     [2C[47m      [1C      [1C  [3C  [2C    [2C  [2C  [40m"
"[1;30;44m        [40m [47m.[37m    [30m.[40m   [44m    [40m"
"[1;30;44m       [40m [47m        [40m    [44m  [40m[2C[1;37;40m"
"[1;30;44m       [40m [47m         [40m    [44m [40m[2C[1;37;40m[0m"
"[1;30;44m       [40m [47m         [40m    [30;44m [40m[2C[1;37;40m[0m"
"[1;30;44m      [40m [47m          [40m     [2C[1;37;40mPingus V.0.0.8[0m"
"[1;30;44m     [40m  [47m          [40m     [2C[1;37;40m[0m"
"[1;30;44m     [43m   [30;47m        [43m   [40m"
"[1;44m   [43m     [47m        [43m    [40m"
"[1;44m [43m       [47m        [43m      [40m"
"[1;44m [43m       [40m [47m      [40m [43m      [40m"
"[0m[255D" << endl;
#endif
  PingusMain::check_args(argc, argv);

  if (!pingus_data_dir)
    pingus_data_dir = getenv("PINGUS_DATA_DIR");

  if (!pingus_data_dir) {
    pingus_data_dir = strdup("/usr/local/share/pingus/");
  }

  pingus_datafile  = new char[255];
  pingus_scriptfile = new char[255];
  
  //cout << "Pingus Data File: " << pingus_datafile << endl;

  strcpy(pingus_datafile, pingus_data_dir);
  strcpy(pingus_scriptfile, pingus_data_dir);

  strcat(pingus_datafile, "pingus.dat");
  strcat(pingus_scriptfile, "pingus.scr");

  cout << "Pingus Data File:   " << pingus_datafile << endl;
  cout << "Pingus Scrpit File: " << pingus_scriptfile << endl;

  if (!levelfile) {
    levelfile = new char[strlen(pingus_data_dir) +
			strlen("level3.plf") + 1];
    strcpy(levelfile, pingus_data_dir);
    strcat(levelfile, "level3.plf");
  }

  cout << "Pingus Level File: " << levelfile << endl;

  // initialisation ClanLib
  CL_System::init_display();
  CL_Display::set_videomode(640, 480, 16);

  load_data();
  srand(time(NULL));
  
  if (!Mouse::init()) {
    cout << "No Mouse Support available" << endl;
  }
}

int
PingusMain::main(int argc, char** argv)
{
  cout << "Pingus " << VERSION << " starting..." << endl;

  init(argc, argv);  
  intro();
  play_level(levelfile);

  cout << "Pingus exit normaly" << endl;
  return 0;
}

/* EOF */
